package trains.tests;

import static org.junit.Assert.assertEquals;

import java.util.ArrayList;
import java.util.SortedSet;
import java.util.TreeSet;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;

import trains.model.DayOfWeek;
import trains.model.InvalidStationException;
import trains.model.Station;
import trains.model.Stop;
import trains.model.Time;
import trains.model.Train;

public class TrainTest {
	private Train train;

	@Before
	public void setUp() throws Exception {
		train = new Train("R", "4242");
	}

	private void checkWrongTrackException(Station s1, Station s2)
	{
		boolean ok = false;
		try
		{
			train.travelDuration(s1,s2);
		}
		catch (InvalidStationException e)
		{
			ok = true;
		}
		Assert.assertTrue(ok);
	}

	@Test
	public void testTravelDuration() throws InvalidStationException {
		SortedSet<Stop> stops = new TreeSet<Stop>();

		Stop orig[] = new Stop[] {
			new Stop(new Station("Bologna", "centrale"), null, new Time(10,20)),
			new Stop(new Station("Parma", ""), new Time(11,10), new Time(11,20)),
			new Stop(new Station("Milano", "centrale"), new Time(12,10), null)
		};
		
		for (Stop s : orig)
		{
			stops.add(s);
		}

		train.setStops(stops);

		assertEquals(new Time(0,50), train.travelDuration(orig[0].getStation(), orig[1].getStation()));
		assertEquals(new Time(0,50), train.travelDuration(orig[1].getStation(), orig[2].getStation()));
		assertEquals(new Time(1,50), train.travelDuration(orig[0].getStation(), orig[2].getStation()));

		checkWrongTrackException(orig[1].getStation(), orig[0].getStation());
		checkWrongTrackException(orig[2].getStation(), orig[0].getStation());
		checkWrongTrackException(orig[2].getStation(), orig[1].getStation());
		checkWrongTrackException(orig[1].getStation(), null);
		checkWrongTrackException(null, orig[1].getStation());

		checkWrongTrackException(orig[1].getStation(), new Station("Milano","lambrate"));
		checkWrongTrackException( new Station("Napoli","centrale"), orig[1].getStation());
	}

	@Test
	public void testGetType() {
		assertEquals("R", train.getType());
	}

	@Test
	public void testGetNumber() {
		assertEquals("4242", train.getNumber());
	}

	@Test
	public void testGetSetDays() {
		ArrayList<DayOfWeek> days = new ArrayList<DayOfWeek>();
		days.add(DayOfWeek.MONDAY);
		days.add(DayOfWeek.WEDNESDAY);
		days.add(DayOfWeek.FRIDAY);

		train.setDays(days);
		days = train.getDays();

		assertEquals(3, days.size());
		assertEquals(DayOfWeek.MONDAY, days.get(0));
		assertEquals(DayOfWeek.WEDNESDAY, days.get(1));
		assertEquals(DayOfWeek.FRIDAY, days.get(2));
	}

	@Test
	public void testGetSetStops() {
		SortedSet<Stop> stops = new TreeSet<Stop>();

		Stop orig[] = new Stop[] {
			new Stop(new Station("Bologna", "centrale"), null, new Time(10,20)),
			new Stop(new Station("Parma", ""), new Time(11,10), new Time(11,20)),
			new Stop(new Station("Milano", "centrale"), new Time(12,10), null)
		};
		
		for (Stop s : orig)
		{
			stops.add(s);
		}

		train.setStops(stops);
		stops = train.getStops();

		assertEquals(orig.length, stops.size());

		assertEquals(orig[0].getDeparture(), stops.first().getDeparture());
		assertEquals(orig[0].getArrival(), stops.first().getArrival());
		assertEquals(orig[0].getStation(), stops.first().getStation());

		assertEquals(orig[orig.length-1].getDeparture(), stops.last().getDeparture());
		assertEquals(orig[orig.length-1].getArrival(), stops.last().getArrival());
		assertEquals(orig[orig.length-1].getStation(), stops.last().getStation());
	}

}
